@@ -63,6 +63,25 @@
 #' L.matrix = UnMethylatedOxBS_sim, M.matrix = MethylatedOxBS_sim,
 #' G.matrix = UnMethylatedTAB_sim, H.matrix = MethylatedTAB_sim)
 #'
+#'
+#' # Example of datasets with zero counts and missing values:
+#'
+#' MethylatedBS_sim[1,1] <- 0
+#' MethylatedOxBS_sim[1,1] <- 0
+#' MethylatedTAB_sim[1,1] <- 0
+#' UnMethylatedBS_sim[1,1] <- 0
+#' UnMethylatedOxBS_sim[1,1] <- 0
+#' UnMethylatedTAB_sim[1,1] <- 0
+#'
+#' MethylatedBS_sim[2,2] <- NA
+#' MethylatedOxBS_sim[2,2] <- NA
+#' MethylatedTAB_sim[2,2] <- NA
+#' UnMethylatedBS_sim[2,2] <- NA
+#' UnMethylatedOxBS_sim[2,2] <- NA
+#' UnMethylatedTAB_sim[2,2] <- NA
+#'
+#'
+#'
 #' @author
 #' Samara Kiihl samara@ime.unicamp.br;
 #' Maria Jose Garrido;
@@ -76,22 +95,22 @@
 #'   doi:10.1093/bioinformatics/btt459.
 #'
 #'   Ayer M, Brunk HD, Ewing GM, Reid WT, Silverman E. An Empirical Distribution Function
-#'   for Sampling with Incomplete Information. Ann. Math. Statist. 1955, 26(4), 641–647.
+#'   for Sampling with Incomplete Information. Ann. Math. Statist. 1955, 26(4), 641-647.
 #'   doi:10.1214/aoms/1177728423.
 #'
 #'   Zongli Xu, Jack A. Taylor, Yuet-Kin Leung, Shuk-Mei Ho, Liang Niu;
 #'   oxBS-MLE: an efficient method to estimate 5-methylcytosine and 5-hydroxymethylcytosine
 #'   in paired bisulfite and oxidative bisulfite treated DNA,
-#'   Bioinformatics, 2016;32(23):3667–3669.
+#'   Bioinformatics, 2016;32(23):3667-3669.
 
 MLML <- function(G.matrix       = NULL,
-                 H.matrix       = NULL,
-                 L.matrix       = NULL,
-                 M.matrix       = NULL,
-                 T.matrix       = NULL,
-                 U.matrix       = NULL,
-                 iterative = FALSE,
-                 tol=0.00001)
+                  H.matrix       = NULL,
+                  L.matrix       = NULL,
+                  M.matrix       = NULL,
+                  T.matrix       = NULL,
+                  U.matrix       = NULL,
+                  iterative = FALSE,
+                  tol=0.00001)
 {
   g <- if (is.null(G.matrix)) {
     NULL
@@ -130,7 +149,7 @@ MLML <- function(G.matrix       = NULL,
 
   six <-function(x1,x2,x3,x4,x5,x6){
     return(identical(x1,x2) & identical(x1,x3) & identical(x1,x4) &
-           identical(x1,x5) & identical(x1,x6))
+             identical(x1,x5) & identical(x1,x6))
   }
 
   pme <- 0.3
@@ -146,9 +165,9 @@ MLML <- function(G.matrix       = NULL,
       !is.null(T.matrix) & !is.null(U.matrix)) {
     ######### 3 methods
 
-      if (!six(rownames(L.matrix),rownames(M.matrix),rownames(T.matrix),
-                        rownames(U.matrix),rownames(G.matrix),rownames(H.matrix)))
-          {stop("Row names are inconsistent")}
+    if (!six(rownames(L.matrix),rownames(M.matrix),rownames(T.matrix),
+             rownames(U.matrix),rownames(G.matrix),rownames(H.matrix)))
+    {stop("Row names are inconsistent")}
     else {
       if (!iterative)
       {
@@ -175,19 +194,19 @@ MLML <- function(G.matrix       = NULL,
         phe <- ph0 + lam*Vh
       } else {
 
-    while (diff > tol) {
-      pme_ant <- pme
-      phe_ant <- phe
-      k <- t * (pme / (pme + phe + 1e-08))
-      j <- g * (pme / (1 - phe + 1e-08))
-      pme <- (m + j + k) / (m + l + h + g + u + t)
-      phe <- ((h - k + t) / (h + g + t + u - j - k)) * (1 - pme)
-      diff_pme <- abs(max(pme - pme_ant))
-      diff_phe <- abs(max(phe - phe_ant))
-      diff <- abs(max(diff_pme, diff_phe))
-    }
+        while (diff > tol) {
+          pme_ant <- pme
+          phe_ant <- phe
+          k <- t * (pme / (pme + phe + 1e-08))
+          j <- g * (pme / (1 - phe + 1e-08))
+          pme <- (m + j + k) / (m + l + h + g + u + t)
+          phe <- ((h - k + t) / (h + g + t + u - j - k)) * (1 - pme)
+          diff_pme <- abs(max(pme - pme_ant,na.rm=TRUE))
+          diff_phe <- abs(max(phe - phe_ant,na.rm=TRUE))
+          diff <- abs(max(diff_pme, diff_phe,na.rm=TRUE))
+        }
       }
-}
+    }
 
     methods <-
       c("TAB-conversion +  oxBS-conversion + standard BS-conversion")
@@ -196,89 +215,89 @@ MLML <- function(G.matrix       = NULL,
   } else if (is.null(G.matrix) || is.null(H.matrix)) {
     ##### oxBS-seq + BS-seq
 
-if (!four(rownames(L.matrix),rownames(M.matrix),rownames(T.matrix),
-                        rownames(U.matrix))){stop("Row names are inconsistent")}
+    if (!four(rownames(L.matrix),rownames(M.matrix),rownames(T.matrix),
+              rownames(U.matrix))){stop("Row names are inconsistent")}
     else {
 
-    if (!iterative)
-    {
-      pme = ifelse(t / (t + u) >= m / (m + l) , m / (m + l) ,
-          (m + t) / (m + l + u + t))
-      phe = ifelse(t / (t + u) >= m / (m + l), t / (t + u) - m / (m + l) , 0)
-    } else {
-      while (diff > tol) {
-        pme_ant <- pme
-        phe_ant <- phe
-        k <- t * (pme / (pme + phe + 1e-08))
-        pme <- (m + k) / (m + l + u + t)
-        phe <- ((-k + t) / (t + u - k)) * (1 - pme)
-        diff_pme <- abs(max(pme - pme_ant))
-        diff_phe <- abs(max(phe - phe_ant))
-        diff <- abs(max(diff_pme, diff_phe))
+      if (!iterative)
+      {
+        pme = ifelse(t / (t + u) >= m / (m + l) , m / (m + l) ,
+                     (m + t) / (m + l + u + t))
+        phe = ifelse(t / (t + u) >= m / (m + l), t / (t + u) - m / (m + l) , 0)
+      } else {
+        while (diff > tol) {
+          pme_ant <- pme
+          phe_ant <- phe
+          k <- t * (pme / (pme + phe + 1e-08))
+          pme <- (m + k) / (m + l + u + t)
+          phe <- ((-k + t) / (t + u - k)) * (1 - pme)
+          diff_pme <- abs(max(pme - pme_ant,na.rm=TRUE))
+          diff_phe <- abs(max(phe - phe_ant,na.rm=TRUE))
+          diff <- abs(max(diff_pme, diff_phe,na.rm=TRUE))
+        }
+
       }
 
     }
 
-}
-
     methods <- c("oxBS-conversion + standard BS-conversion")
 
 
   } else if (is.null(M.matrix) || is.null(L.matrix)) {
     ##### TAB-seq + BS-seq
 
-       if (!four(rownames(G.matrix),rownames(H.matrix),rownames(T.matrix),
-                        rownames(U.matrix))){stop("Row names are inconsistent")}
+    if (!four(rownames(G.matrix),rownames(H.matrix),rownames(T.matrix),
+              rownames(U.matrix))){stop("Row names are inconsistent")}
     else {
 
-    if (!iterative)
-    {
-      pme = ifelse(t / (t + u) >= h / (g + h) , t / (t + u) - h / (g + h) , 0)
-      phe = ifelse(t / (t + u) >= h / (g + h) , h / (g + h),
-          (h + t) / (g + h + t + u))
-    } else {
-      while (diff > tol) {
-        pme_ant <- pme
-        phe_ant <- phe
-        k <- t * (pme / (pme + phe + 1e-08))
-        j <- g * (pme / (1 - phe + 1e-08))
+      if (!iterative)
+      {
+        pme = ifelse(t / (t + u) >= h / (g + h) , t / (t + u) - h / (g + h) , 0)
+        phe = ifelse(t / (t + u) >= h / (g + h) , h / (g + h),
+                     (h + t) / (g + h + t + u))
+      } else {
+        while (diff > tol) {
+          pme_ant <- pme
+          phe_ant <- phe
+          k <- t * (pme / (pme + phe + 1e-08))
+          j <- g * (pme / (1 - phe + 1e-08))
 
-        pme <- (j + k) / (g + h + t + u)
-        phe <- ((h - k + t) / (g + h + t + u - k - j)) * (1 - pme)
-        diff_pme <- abs(max(pme - pme_ant))
-        diff_phe <- abs(max(phe - phe_ant))
-        diff <- abs(max(diff_pme, diff_phe))
+          pme <- (j + k) / (g + h + t + u)
+          phe <- ((h - k + t) / (g + h + t + u - k - j)) * (1 - pme)
+          diff_pme <- abs(max(pme - pme_ant,na.rm=TRUE))
+          diff_phe <- abs(max(phe - phe_ant,na.rm=TRUE))
+          diff <- abs(max(diff_pme, diff_phe,na.rm=TRUE))
+        }
       }
     }
-}
     methods <- c("TAB-conversion + standard BS-conversion")
 
 
   } else {
     ##### TAB-seq + Ox-seq
 
-      if (!four(rownames(G.matrix),rownames(H.matrix),rownames(M.matrix),
-                        rownames(L.matrix))){stop("Row names are inconsistent")}
+    if (!four(rownames(G.matrix),rownames(H.matrix),rownames(M.matrix),
+              rownames(L.matrix))){stop("Row names are inconsistent")}
     else {
 
-    if (!iterative)
-    {
-      pme = ifelse(m/(m+l)<=g/(h+g),m/(m+l),(g+m)/(g+h+m+l))
-      phe = ifelse(m/(m+l)<=g/(h+g),h/(h+g),(h+l)/(g+h+m+l))
-    } else {
-      while (diff > tol) {
-        pme_ant <- pme
-        phe_ant <- phe
-        j <- g * (pme / (1 - phe + 1e-08))
+      if (!iterative)
+      {
+        pme = ifelse(m/(m+l)<=g/(h+g),m/(m+l),(g+m)/(g+h+m+l))
+        phe = ifelse(m/(m+l)<=g/(h+g),h/(h+g),(h+l)/(g+h+m+l))
+      } else {
+        while (diff > tol) {
+          pme_ant <- pme
+          phe_ant <- phe
+          j <- g * (pme / (1 - phe + 1e-08))
 
-        pme <- (j + m) / (g + h + m + l)
-        phe <- ((h) / (h + g - j)) * (1 - pme)
-        diff_pme <- abs(max(pme - pme_ant))
-        diff_phe <- abs(max(phe - phe_ant))
-        diff <- abs(max(diff_pme, diff_phe))
+          pme <- (j + m) / (g + h + m + l)
+          phe <- ((h) / (h + g - j)) * (1 - pme)
+          diff_pme <- abs(max(pme - pme_ant,na.rm=TRUE))
+          diff_phe <- abs(max(phe - phe_ant,na.rm=TRUE))
+          diff <- abs(max(diff_pme, diff_phe,na.rm=TRUE))
+        }
       }
     }
-}
     methods <- c("TAB-conversion +  oxBS-conversion")
 
 
@@ -287,6 +306,10 @@ if (!four(rownames(L.matrix),rownames(M.matrix),rownames(T.matrix),
 
   pm <- pme
   ph <- phe
+
+  pm[is.nan(pm)] <- NA
+  ph[is.nan(ph)] <- NA
+
   pu <- (1 - pm - ph)
 
   proportions <- list()
